//
//  AttributedStringOperator.swift
//  WolfStrings
//
//  Created by Wolf McNally on 6/24/17.
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in all
//  copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
//  SOFTWARE.

#if !os(Linux)
import Foundation

///
/// Attributed-String-Operator
///
/// The special character here ("§") is called the "section marker" and is typed by pressing Option-6.
///
postfix operator §

public postfix func § (left: String) -> AttributedString {
    return AttributedString(string: left)
}

public postfix func § (left: AttributedString) -> AttributedString {
    return left.mutableCopy() as! AttributedString
}

public postfix func § (left: NSAttributedString) -> AttributedString {
    return left.mutableCopy() as! AttributedString
}

postfix operator §?

public postfix func §? (left: String?) -> AttributedString? {
    guard let left = left else { return nil }
    return AttributedString(string: left)
}

public postfix func §? (left: AttributedString?) -> AttributedString? {
    guard let left = left else { return nil }
    return left.mutableCopy() as? AttributedString
}

public postfix func §? (left: NSAttributedString?) -> AttributedString? {
    guard let left = left else { return nil }
    return left.mutableCopy() as? AttributedString
}

public func toAttributedString(_ string: String) -> AttributedString {
    return AttributedString(string: string)
}
#endif
